
yarip.Control = function(overlay)
{
	this.overlay = overlay ? overlay : null;
	this.prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	this.prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

	this.getValue = function(preference, defaultValue, type)
	{
		try
		{
			switch(type)
			{
				case yarip.BOOLEAN:
					return this.prefBranch.getBoolPref(preference);

				case yarip.INTEGER:
					return this.prefBranch.getIntPref(preference);

				case yarip.STRING:
					return this.prefBranch.getCharPref(preference);
			}
		}
		catch(e)
		{
			if (!defaultValue) return null;
			this.setValue(preference, defaultValue, type);
		}
		return this.getValue(preference, null, type);
	}

	this.setValue = function(preference, value, type)
	{
		try
		{
			switch(type)
			{
				case yarip.BOOLEAN:
					this.prefBranch.setBoolPref(preference, "" + value == "true");
					break;

				case yarip.INTEGER:
					this.prefBranch.setIntPref(preference, value);
					break;

				case yarip.STRING:
					this.prefBranch.setCharPref(preference, value);
					break;

				case yarip.RESET:
					this.prefBranch.clearUserPref(preference);
			}
		}
		catch(e){}
	}

	this.update = function(type)
	{
		if (!type) return;

		try
		{
			if (!this.overlay) return;

			switch(type)
			{
				case yarip.UPDATE_ENABLED:
					this.overlay.updateEnabled();
			}
		}
		catch(e){}
	}

	this.loadPages = function()
	{
		file = this.getFile(yarip.XPATH);

		var doc = this.getDoc(file);
		if (!doc || doc.documentElement.nodeName == "parsererror") return null;

		var pages = {};
		var iPage = doc.evaluate("./yarip/page", doc, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
		var nPage = iPage.iterateNext();
		while (nPage)
		{
			var pName = nPage.getAttribute("name");
			if (!pages[pName]) pages[pName] = new yarip.Page(pName);

			// whitelist
			var iWhitelist = doc.evaluate("./whitelist", nPage, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var nWhitelist = iWhitelist.iterateNext();
			if (nWhitelist)
			{
				var iXPath = doc.evaluate("./xpath", nWhitelist, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
				var nXPath = iXPath.iterateNext();
				while (nXPath)
				{
					pages[pName].whitelist.addXPath(new yarip.XPath(
						nXPath.getAttribute("value"),
						nXPath.getAttribute("deep"),
						nXPath.getAttribute("found"),
						nXPath.getAttribute("notFound"),
						nXPath.getAttribute("notRemoved")));
					nXPath = iXPath.iterateNext();
				}
			}

			// blacklist
			var iBlacklist = doc.evaluate("./blacklist", nPage, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var nBlacklist = iBlacklist.iterateNext();
			if (nBlacklist)
			{
				var iXPath = doc.evaluate("./xpath", nBlacklist, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
				var nXPath = iXPath.iterateNext();
				while (nXPath)
				{
					pages[pName].blacklist.addXPath(new yarip.XPath(
						nXPath.getAttribute("value"),
						nXPath.getAttribute("deep"),
						nXPath.getAttribute("found"),
						nXPath.getAttribute("notFound"),
						nXPath.getAttribute("notRemoved")));
					nXPath = iXPath.iterateNext();
				}
			}

			nPage = iPage.iterateNext();
		}

		return pages;
	}

	this.savePages = function(pages)
	{
		file = this.getFile(yarip.XPATH);

		var aPage = [];
		for (var pName in pages)
		{
			if (!pages[pName]) continue;
			pages[pName].whitelist.sortXPaths();
			pages[pName].blacklist.sortXPaths();
			aPage.push(pages[pName]);
		}
		aPage.sort(yarip.PageSorter);

		pages = {};
		for (var i = 0; i < aPage.length; i++) pages[aPage[i].name] = aPage[i];

		var data = this.generateXml(pages);
		this.saveToFile(data, file);
	}

	this.generateXml = function(pages)
	{
		var r = "<?xml version=\"1.0\" encoding=\"" + yarip.CHARSET + "\"?>\n<yarip>\n";
		for (var pName in pages) r += pages[pName].generateXml();
		r += "</yarip>";
		return r;
	}

	this.getDoc = function(file)
	{
		if (!file) return null;
		var data = this.getData(file);
		var parser = new DOMParser();
		return parser.parseFromString(data, "text/xml");
	}

	this.getData = function(file)
	{
		var fiStream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
		fiStream.init(file, 0x01, 00004, null);
		var siStream = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
		siStream.init(fiStream);
		var data = siStream.read(siStream.available());
		var suc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		suc.charset = yarip.CHARSET;
		var data = suc.ConvertToUnicode(data);
		siStream.close();
		fiStream.close();
		return data;
	}

	this.getFile = function(path)
	{
		var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		var aPath = path.split(/\/|\\/);
		for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
		if (!file.exists())
		{
			file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
			this.saveToFile("<?xml version=\"1.0\" encoding=\"" + yarip.CHARSET + "\"?>\n<yarip>\n</yarip>", file);
		}
		if (file.isFile()) return file;
		else return null;
	}

	this.saveToFile = function(data, file)
	{
		if (!data || !file) return;
		var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		fos.init(file, 0x02 | 0x08 | 0x20, 0600, 0);
		var parser = new DOMParser();
		var doc = parser.parseFromString(data, "text/xml");
		var serializer = new XMLSerializer();
		serializer.serializeToStream(doc, fos, yarip.CHARSET);
		fos.close();
	}
}
